#!/bin/bash
#Automatically Created by buildpkg 1.6.1
#Maintainer: Ejis <ejisneos@gmail.com>
#url: http://ccrma.stanford.edu/software/stk/

######Begin Redundant Code######################################
check_for_root() {
	if [ "$UID" != "0" ]; then
		echo "You need to be root"
		exit 1
	fi
}

clean_dirs () {
        for COMPLETED in src pkg; do
                if [ -e $COMPLETED ]; then
                        rm -rf $COMPLETED
                fi
        done
}

clean_old_builds () {
	for COMPLETED in $package.{tgz,md5,dep}; do
		if [ "$KEEPDEPFILE" = "1" ] && [ $COMPLETED = $package.dep ]; then
			continue
		elif [ -e $COMPLETED ]; then
			rm -rf $COMPLETED
		fi
	done
	clean_dirs
}

set_pre_permissions() {
	cd $startdir/src
	find . -perm 664 -exec chmod 644 {} \;
	find . -perm 600 -exec chmod 644 {} \;
	find . -perm 444 -exec chmod 644 {} \;
	find . -perm 400 -exec chmod 644 {} \;
	find . -perm 440 -exec chmod 644 {} \;
	find . -perm 777 -exec chmod 755 {} \;
	find . -perm 775 -exec chmod 755 {} \;
	find . -perm 511 -exec chmod 755 {} \;
	find . -perm 711 -exec chmod 755 {} \;
	find . -perm 555 -exec chmod 755 {} \;
}


check_for_menu_compliance() {
	if [ -d "$startdir/pkg/usr/share/applications" ]; then
		cd $startdir/pkg/usr/share/applications
		for desktop in *.desktop; do
			Icon=$(grep -e "^Icon=" $desktop | sed 's/Icon=//')
			if [[ "$Icon" ]]; then
				if [ ! "$Icon" == "${Icon%.*}" ]; then
					sed '/Icon/s/\..*$//' -i $desktop
					Icon=$(grep -e "^Icon=" $desktop | sed 's/Icon=//')
				fi
				if [ -d "$startdir/pkg/usr/share/pixmaps" ]; then
					cd $startdir/pkg/usr/share/pixmaps
					for pixmaps in *; do
						[ -d $pixmaps ] && continue
						[ ! "${pixmaps##*.}" == "png" -a ! "${pixmaps##*.}" == "svg" ] && continue
				 		if [ "${pixmaps%.*}" == "$Icon" ]; then
				 			DIR=$(file $pixmaps | awk -F, '{print $2}' | sed 's/ //g')
							[ o"$DIR" == "o" ] && DIR="scalable"
							mkdir -p $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							mv $pixmaps $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							if [[ ! "$(ls $startdir/pkg/usr/share/pixmaps)" ]]; then
								rm -rf $startdir/pkg/usr/share/pixmaps
							fi
						fi
					done
				fi
				if [ -d "$startdir/pkg/usr/share/icons" ]; then
					cd $startdir/pkg/usr/share/icons
					for icons in *; do
				 		if [ "${icons%.*}" == "$Icon" ]; then
							[ ! "${pixmaps##*.}" == "png" -a ! "${pixmaps##*.}" == "svg" ] && continue
				 			DIR=$(file $icons | awk -F, '{print $2}' | sed 's/ //g')
							[ o"$DIR" == "o" ] && DIR="scalable"
							mkdir -p $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							mv $icons $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
						fi
					done
				fi
			fi
		done
	fi
}


gzip_man_and_info_pages() {
	for DOCS in man info; do
		if [ -d "$startdir/pkg/usr/share/$DOCS" ]; then
			mv $startdir/pkg/usr/share/$DOCS $startdir/pkg/usr/$DOCS
			if [[ ! "$(ls $startdir/pkg/usr/share)" ]]; then
				rm -rf $startdir/pkg/usr/share
			fi
		fi
		if [ -d "$startdir/pkg/usr/$DOCS" ]; then
			# I've never seen symlinks in info pages....
			if [ "$DOCS" == "man" ]; then
				(cd $startdir/pkg/usr/$DOCS
				for manpagedir in $(find . -type d -name "man*" 2> /dev/null) ; do
					( cd $manpagedir
					for eachpage in $( find . -type l -maxdepth 1 2> /dev/null) ; do
						ln -s $( readlink $eachpage ).gz $eachpage.gz
						rm $eachpage
					done )
				done)
			fi
			find $startdir/pkg/usr/$DOCS -type f -exec gzip -9 '{}' \;
		fi
	done
}

set_post_permissions() {
	for DIRS in usr/share/icons usr/doc; do
		if [ -d "$startdir/pkg/$DIRS" ]; then
			if [ "$DIRS" == "usr/doc" ]; then
				find $startdir/pkg/$DIRS -type f -exec chmod 644 {} \;
				find $startdir/pkg/$DIRS -type d -exec chmod 755 {} \;
			fi
		fi
		[ -d $startdir/pkg/$DIRS ] && chown root:root -R $startdir/pkg/$DIRS
	done
	[ -d $startdir/pkg/usr/bin ] && find $startdir/pkg/usr/bin -user root -group bin -exec chown root:root {} \;
}

copy_build_script() {
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver/
	cp $startdir/build-$pkgname.sh $startdir/pkg/usr/src/$pkgname-$pkgver/build-$pkgname.sh
	[ -f $startdir/ZENBUILD ] && cp $startdir/ZENBUILD $startdir/pkg/usr/src/$pkgname-$pkgver/ZENBUILD
}

create_package() {
	ls -lR $startdir/pkg
	cd $startdir/pkg
	/sbin/makepkg -l y -c n $startdir/$package.tgz
	cd $startdir
	md5sum $package.tgz > $startdir/$package.md5
}

strip_binaries() {
	cd $startdir/pkg
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
}
#########End Redundant Code#####################################
#########Begin Non Redundant Code##############################

prepare_directory() {
	mkdir $startdir/src
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver
	for SOURCES in ${source[@]}; do
		protocol=$(echo $SOURCES | sed 's|:.*||')
	        file=$(basename $SOURCES | awk -F= '{print $NF}')
		if [ ! -f "$file" ]; then
			if [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
				echo -e "\nDownloading $(basename $SOURCES)\n"
        	                wget $SOURCES -O $file
				if [ ! "$?" == "0" ]; then
					echo "Download failed"
					exit 2
				fi 
			else
				echo "$SOURCES does not appear to be a url nor is it in the directory"
				exit 2
			fi
		fi
		cp -R $file $startdir/src
		if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
			cp -R $startdir/$(basename $SOURCES) $startdir/pkg/usr/src/$pkgname-$pkgver/
		fi
		if [ "${SOURCES##*.}" == "desktop" ]; then
			[[ "$(grep -e "^Categories=[a-zA-Z]*;" $(basename $SOURCES))" ]] && install -D $startdir/$(basename $SOURCES) /home/aymeric/InstallLogiciels/MAO/LMMS/stk/Build/pkg/usr/share/applications/$(basename $SOURCES)
		fi
	done
}

extract_source() {
	cd $startdir/src
	if [[ "$(ls $startdir/src)" ]]; then	
		for FILES in ${source[@]}; do
	        	FILES="$(basename $FILES | awk -F= '{print $NF}')"
			file_type=$(file -biz "$FILES")
			unset cmd
			case "$file_type" in
				*application/x-tar*)
					cmd="tar -xf" ;;
				*application/x-zip*)
					cmd="unzip" ;;
				*application/x-gzip*)
					cmd="gunzip -d -f" ;;
				*application/x-bzip*)
					cmd="bunzip2 -f" ;;
			esac
			if [ "$cmd" != "" ]; then
				echo "$cmd $FILES"
	        	        $cmd $FILES
			fi
		done
	elif [ ! "$source" ]; then
		echo -n "" # lame fix
	else
		echo "no files in the src directory $startdir/src"
		exit 2
	fi
}

build() {
        cd $startdir/src/$pkgname-$pkgver
		./configure --with-jack RAWWAVE_PATH='/usr/share/stk/rawwaves/' INCLUDE_PATH='/usr/include/stk/'
		
        cd $startdir/src/$pkgname-$pkgver/src
        make || return 1
		
		
		 mkdir -p /usr/include/stk
		 mkdir -p /usr/share/stk/rawwaves
		 
		 cp -rf $startdir/src/$pkgname-$pkgver/include/* /usr/include/stk/
		 cp -rf $startdir/src/$pkgname-$pkgver/rawwaves/* /usr/share/stk/rawwaves/
		
		 cd $startdir/src/$pkgname-$pkgver/projects/demo/
        make || return 1
		
		 cd $startdir/src/$pkgname-$pkgver/projects/effects/
        make || return 1
		
		 cd $startdir/src/$pkgname-$pkgver/projects/examples/
        make || return 1
		
		 cd $startdir/src/$pkgname-$pkgver/projects/ragamatic/
        make || return 1
		
		 rm -rfv /usr/include/stk /usr/share/stk
		
		
		 mkdir -p $startdir/pkg/usr/lib
		 mkdir -p $startdir/pkg/usr/doc/stk-$pkgver/html
		 mkdir -p $startdir/pkg/usr/include/stk
		 mkdir -p $startdir/pkg/usr/share/stk/rawwaves
		 mkdir -p $startdir/pkg/usr/share/stk/tcl/demo
		 mkdir -p $startdir/pkg/usr/share/stk/tcl/effects
		 mkdir -p $startdir/pkg/usr/share/stk/tcl/ragamatic
		 mkdir -p $startdir/pkg/usr/bin
		 
		 
		 cp $startdir/src/$pkgname-$pkgver/src/libstk.a $startdir/pkg/usr/lib/
		 
		 cp -rf $startdir/src/$pkgname-$pkgver/doc/html $startdir/pkg/usr/doc/stk-$pkgver/
		 cp $startdir/src/$pkgname-$pkgver/doc/README-Linux.txt $startdir/pkg/usr/doc/stk-$pkgver/
		 cp $startdir/src/$pkgname-$pkgver/doc/ReleaseNotes.txt $startdir/pkg/usr/doc/stk-$pkgver/
		 cp $startdir/src/$pkgname-$pkgver/doc/SKINI.txt $startdir/pkg/usr/doc/stk-$pkgver/
		 cp $startdir/src/$pkgname-$pkgver/doc/hierarchy.txt $startdir/pkg/usr/doc/stk-$pkgver/
		 
		 cp -rf $startdir/src/$pkgname-$pkgver/include/* $startdir/pkg/usr/include/stk/
		 
		 cp -rf $startdir/src/$pkgname-$pkgver/rawwaves/* $startdir/pkg/usr/share/stk/rawwaves/
		 
		 cp -rf $startdir/src/$pkgname-$pkgver/projects/demo/tcl/* $startdir/pkg/usr/share/stk/tcl/demo/
		 cp -rf $startdir/src/$pkgname-$pkgver/projects/effects/tcl/* $startdir/pkg/usr/share/stk/tcl/effects/
		 cp -rf $startdir/src/$pkgname-$pkgver/projects/ragamatic/tcl/* $startdir/pkg/usr/share/stk/tcl/ragamatic/
		 
		 cp $startdir/src/$pkgname-$pkgver/projects/demo/demo $startdir/pkg/usr/bin/stk-demo
		 cp $startdir/src/$pkgname-$pkgver/projects/effects/effects $startdir/pkg/usr/bin/stk-effects
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/examples $startdir/pkg/usr/bin/stk-examples-sine
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/sine $startdir/pkg/usr/bin/stk-examples-sineosc
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/foursine $startdir/pkg/usr/bin/stk-examples-foursine
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/play $startdir/pkg/usr/bin/stk-examples-play
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/record $startdir/pkg/usr/bin/stk-examples-record
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/audioprobe $startdir/pkg/usr/bin/stk-examples-audioprobe
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/midiprobe $startdir/pkg/usr/bin/stk-examples-midiprobe
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/duplex $startdir/pkg/usr/bin/stk-examples-duplex
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/inetIn $startdir/pkg/usr/bin/stk-examples-inetIn
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/inetOut $startdir/pkg/usr/bin/stk-examples-inetOut
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/rtsine $startdir/pkg/usr/bin/stk-examples-rtsine
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/crtsine $startdir/pkg/usr/bin/stk-examples-crtsine
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/bethree $startdir/pkg/usr/bin/stk-examples-bethree
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/controlbee $startdir/pkg/usr/bin/stk-examples-controlbee
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/threebees $startdir/pkg/usr/bin/stk-examples-threebees
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/playsmf $startdir/pkg/usr/bin/stk-examples-playsmf
		 cp $startdir/src/$pkgname-$pkgver/projects/examples/grains $startdir/pkg/usr/bin/stk-examples-grains
		 cp $startdir/src/$pkgname-$pkgver/projects/ragamatic/ragamat $startdir/pkg/usr/bin/stk-ragamat
}

create_slackdesc() {
mkdir $startdir/pkg/install
cat <<"EODESC" >$startdir/pkg/install/slack-desc
stk: stk - Synthesis ToolKit
stk: 
stk: The Synthesis ToolKit in C++ (STK) is a set of open source
stk: audio signal processing and algorithmic synthesis classes
stk: written in the C++ programming language. STK was designed to
stk: facilitate rapid development of music synthesis and audio
stk: processing software, with an emphasis on cross-platform
stk: functionality, realtime control, ease of use, and educational
stk: example code.
stk: 
stk: 
EODESC
}

copy_docs() {
	for stuff in ${docs[@]}; do
		if [ ! -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
			mkdir -p $startdir/pkg/usr/doc/$pkgname-$pkgver
		fi
		find $startdir/src -type f -iname "$stuff" -exec cp -R '{}' $startdir/pkg/usr/doc/$pkgname-$pkgver \;
	done
}
detect_dependencies() {
	echo "Detecting dependencies..."
	depends=$(requiredbuilder -z $startdir/$package.tgz)
	
	depstring=$depends
	unset FIRST
	for deps in $(echo "${depstring}" | sed 's/,/\n/g' | sort -u); do
		if [[ ! "$FIRST" ]]; then
			finaldeps=$deps,
			FIRST='1'
			continue
		fi
		finaldeps=${finaldeps}${deps},
	done
	echo "$finaldeps" | sed -e 's/^,//' -e 's/,$//' > $startdir/$package.dep
}
create_source_file(){
	[ -f $package.src ] && rm $package.src
	if [ ! -z $sourcetemplate ]; then
		echo $sourcetemplate/ZENBUILD >> $package.src
		echo $sourcetemplate/build-$pkgname.sh >> $package.src
	else
		echo ZENBUILD >> $package.src
		echo build-$pkgname.sh >> $package.src
	fi
	for SOURCES in ${source[@]}; do
		protocol=$(echo $SOURCES | sed 's|:.*||')
		if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
			if [ ! -z $sourcetemplate ]; then
				echo $sourcetemplate/$(basename $SOURCES) >> $package.src
			else
				echo $(basename $SOURCES) >> $package.src
			fi
		else
			echo $SOURCES >> $package.src
		fi
	done
}
post_checks(){
	# Ideas taken from src2pkg :)
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		for DIRS in usr/doc/$pkgname-$pkgver usr/doc; do
			cd $startdir/pkg/$DIRS
			if [[ $(find . -type f) = "" ]] ; then
				cd ..
				rmdir $DIRS
			fi
		done
	fi
	# if the docs weren't deleted ...
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		cd $startdir/pkg/usr/doc/$pkgname-$pkgver
		#remove zero lenght files
		if [[ $(find . -type f -size 0) ]]; then
			echo "Removing some zero lenght files"
			find . -type f -size 0 -exec rm -f {} \;
		fi
	fi
	# check if we need to add code to handle info pages
	if [[ -d $startdir/pkg/usr/info ]] && [[ ! $(grep install-info $startdir/pkg/install/doinst.sh &> /dev/null) ]] ; then
		echo "Found info files - Adding install-info command to doinst.sh"
		INFO_LIST=$(ls -1 $startdir/pkg/usr/info)
		echo "" >> $startdir/pkg/install/doinst.sh
		echo "if [ -x usr/bin/install-info ] ; then" >> $startdir/pkg/install/doinst.sh
		for page in $(echo $INFO_LIST) ; do
			echo " usr/bin/install-info --info-dir=usr/info usr/info/$page 2>/dev/null" >> $startdir/pkg/install/doinst.sh
		done
		echo "fi" >> $startdir/pkg/install/doinst.sh
	fi
	[[ -e $startdir/pkg/usr/info/dir ]] && rm -f $startdir/pkg/usr/info/dir

	if [ -d $startdir/pkg/etc ]; then
		cd $startdir/pkg/
		for conf in $(find ./etc -type f) ; do
			conf=${conf: 2}
			dotnew=( "${dotnew[@]}" "$conf" )
		done
	fi
	if [[ "$dotnew" ]]; then
        for files in ${dotnew[@]} ; do
                fullfile="${startdir}/pkg/${files}"
                if [ -e "$fullfile" ]; then
                        mv $fullfile ${fullfile}.new
                else
                        echo "$fullfile was not found"
                        exit 2
                fi
        done
        cat<<"EODOTNEW" >>$startdir/pkg/install/doinst.sh
#Added by buildpkg 1.6.1
dotnew() {
        NEW="${1}.new"
        OLD="$1"
        if [ ! -e $OLD ]; then
                mv $NEW $OLD
        elif [ "$(cat $OLD | md5sum)" = "$(cat $NEW | md5sum)" ]; then
                rm $NEW
        fi
}
EODOTNEW
for i in ${dotnew[@]}; do
echo "dotnew $i" >> $startdir/pkg/install/doinst.sh
done
fi
}

####End Non Redundant Code############################

#Variables

startdir=$(pwd)
SRC=$startdir/src
PKG=$startdir/pkg

pkgname=stk
pkgver=4.3.1
pkgrel=1
zenver=54
arch=i486
package=$pkgname-$pkgver-$arch-$zenver.$pkgrel
source=("http://ccrma.stanford.edu/software/stk/release/stk-4.3.1.tar.gz")
sourcetemplate=http://zenwalk.pinguix.com/user-accounts/ejis/extra/l/stk
docs=(authors copying changelog install news readme)
export CFLAGS="-O2 -march=i486 -mtune=i686"
export CXXFLAGS="-O2 -march=i486 -mtune=i686"

#Execution

check_for_root
clean_old_builds
prepare_directory
extract_source
set_pre_permissions
build
if [ ! "$?" = "0" ]; then
	echo "build() failed."
	exit 2
fi
create_slackdesc
post_checks
copy_docs
check_for_menu_compliance
strip_binaries
gzip_man_and_info_pages
set_post_permissions
copy_build_script
create_package
create_source_file
detect_dependencies
echo "Package has been built."
echo "Cleaning pkg and src directories"
clean_dirs
